#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <string.h>

void writeOctave(int octave, FILE *file)
{
  if (octave < 3)
  {
    for (int i = 0; i < (3-octave); ++i)
    {
      fputs(",", file);
    }
  }
  else
  {
    for (int i = 0; i < (octave-3); ++i)
    {
      fputs("'", file);
    }
  }
}

void writeRhythm(const char* rhythm, FILE *file)
{
  if (strcmp(rhythm, "ts") == 0) { fputs("32", file); }
  else if (strcmp(rhythm, "st") == 0) { fputs("16", file); }
  else if (strcmp(rhythm, "et") == 0) { fputs("8", file); }
  else if (strcmp(rhythm, "qr") == 0) { fputs("4", file); }
  else if (strcmp(rhythm, "hf") == 0) { fputs("2", file); }
  else if (strcmp(rhythm, "wh") == 0) { fputs("1", file); }
}

struct Note{
  char* pitch;
  char* rhythm; 
};
struct Harmony{
    struct Note *arr; 
    int length;
};
struct Melody{
  struct Harmony *arr;
  int length;
};

void writehelper(const char *pitch, const char* rhythm, FILE *file){
    int octave;
    fprintf(file, "%c", tolower(pitch[0]));

    if (pitch[1] == '#')
    {
      fputs("is", file);
      octave = pitch[2] - '0';
    }
    else if (pitch[1] == '@')
    {
      fputs("es", file);
      octave = pitch[2] - '0';
    }
    else
    {
      octave = pitch[1] - '0';
    }
    writeOctave(octave, file);
    writeRhythm(rhythm, file);
    fputs(" ", file);
}

void write(const struct Harmony m, const char *outputFile)
{
  FILE *file;

  char fullName[1024];
  strcpy(fullName, outputFile);
  strcat(fullName, ".ly");

  file = fopen(fullName, "w+");
  fputs("\\version \"2.22.0\"\n\\score {\n{ ", file);
  for (int i= 0; i<m.length; i++){
    writehelper(m.arr[i].pitch, m.arr[i].rhythm, file);
  }
  fputs(" }\n\\midi { \\tempo 4 = 120 }\n}\n", file);
  fclose(file);
}

void writeN(const struct Melody m, const char *outputFile)
{
  FILE *file;

  char fullName[1024];
  strcpy(fullName, outputFile);
  strcat(fullName, ".ly");

  file = fopen(fullName, "w+");
  fputs("\\version \"2.22.0\"\n\\score {\n{ ", file);
  for (int i= 0; i<m.length; i++){
    if(m.arr[i].length==1){
      if (i==m.length-1){
        fputs("\n", file);
      }
      writehelper(m.arr[i].arr[0].pitch, m.arr[i].arr[0].rhythm, file);
    }
    else{
      fputs(" <", file);
      for(int j=0; j<m.arr[i].length;j++){
        writehelper(m.arr[i].arr[j].pitch, "skip:)", file);
      }
      fputs(">", file);
      writeRhythm(m.arr[i].arr[0].rhythm, file);
    }
  }
  fputs(" }\n\\midi { \\tempo 4 = 120 }\n}\n", file);
  fclose(file);
}

void seed() {
  srand ( time(NULL) );
}

int randInt(int min, int max)
{
  return (rand() % (max - min + 1)) + min;
}
